
#define STRICT
#include "D3DAppBase.h"
#include "DIBase.h"

class MD2Viewer:public D3DAppBase{

public:

	MD2ModelObject *md2;
	MediaManager* Media;
	DIManager *Input;
	TextDrop* drop;
	Light* light;

	double theta, phi;
	float zoom;
	DWORD FrameCount;
	BOOL isFlat,isWire,animLoop;

	LPSTR lpCmdLine;

	MD2Viewer();
	~MD2Viewer();
	HRESULT SceneInit();
	HRESULT RenderFrame();
	HRESULT End();
	LRESULT MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

};

MD2Viewer::MD2Viewer():D3DAppBase(){
	
	md2=NULL;
	Media=NULL;
	Input=NULL;
	drop=NULL;
	light=NULL;

	theta=0;
	phi=0;
	zoom=70;
	lpCmdLine="";
	FrameCount=0;
	isFlat=FALSE;
	isWire=FALSE;
	animLoop=TRUE;
	fsBPP=32;
	
}

MD2Viewer::~MD2Viewer(){
	SAFE_DELETE(Media);
	SAFE_DELETE(Input);
	SAFE_DELETE(drop);
	SAFE_DELETE(md2);
}

HRESULT MD2Viewer::SceneInit(){
	HRESULT hr;

	if(Media==NULL){
		Media=new MediaManager(d3dDevice);
		Input=new DIManager();
		drop=new TextDrop(d3dDevice,Media,"Arial",12,400,400);

		Media->MD2Path[0]='\0';

		light=new Light(d3dDevice,D3DLIGHT_POINT,PolarVect(theta,phi,zoom),D3DXVECTOR3(1,1,1),500.0f);
		light->setColors(D3DXCOLOR(1,1,1,1),D3DXCOLOR(0,0,0,0),D3DXCOLOR(0,0,0,0));

		Input->CreateMouse(hWnd,0,DISCL_NONEXCLUSIVE|DISCL_FOREGROUND);
		
		if(FAILED(hr=Media->LoadTextureRes("back",IDB_BACK)))
			return ErrorQuit(hr,"Error loading back texture");

		md2=new MD2ModelObject();
		if(lpCmdLine)
			if(FAILED(hr=md2->Load(d3dDevice,lpCmdLine,"back",NULL,Media)))
				return ErrorQuit(hr,"Error loading MD2 model");
	}

	setAmbient(100,100,100);

	drop->Restore();
	//md2->Restore();
	light->Restore();

	return S_OK;
}

HRESULT MD2Viewer::RenderFrame(){

	DIMOUSESTATE2 dims2;
	Input->GetImmediateMouse(&dims2);

	int difx=dims2.lX,dify=dims2.lY;
	zoom-=dims2.lZ/100;
	if(CHECKSTATE(dims2.rgbButtons[2]))zoom+=dims2.lY;
	if(zoom<=5)zoom=5;

	if(CHECKSTATE(dims2.rgbButtons[0])){
		theta+=difx*0.01;
		if(fabs(phi+dify*0.01)<=D3DX_PI/2)phi+=dify*0.01;
		if(theta>D3DX_PI*2)theta-=D3DX_PI*2;
		if(theta<0)theta+=D3DX_PI*2;
	}		

	setView( D3DXVECTOR3(0,0,0),PolarVect(theta,phi,zoom) );	
	light->light.Position=PolarVect(theta,phi,zoom);
	light->Restore();

	if(FrameCount%3==0){
		md2->PrevFrame=md2->CurrentFrame;
		md2->CurrentFrame=FrameCount/3%md2->Mesh.NumFrames;
	}
	md2->BlendFactor=(float)(FrameCount%3)/3;

	FAILRET(md2->Render());
	
	drop->AddText(lpCmdLine,10,10,RGB(255,255,255));
	drop->AddText(md2->getCurrentFrame()->framename,10,22,RGB(255,255,255));
	FAILRET(drop->Render());

	if(animLoop)FrameCount++;
	return S_OK;
}

HRESULT MD2Viewer::End(){
	drop->Invalidate();
	//md2->Invalidate();
	return S_OK;
}

LRESULT MD2Viewer::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam ){

	switch(uMsg){
	case WM_CHAR:
		
		switch((TCHAR) wParam){
		case 's':
			d3dDevice->SetRenderState(D3DRS_SHADEMODE, (isFlat=!isFlat) ? D3DSHADE_FLAT : D3DSHADE_GOURAUD);
			return 0;
		case 'w':
			d3dDevice->SetRenderState(D3DRS_FILLMODE, (isWire=!isWire) ? D3DFILL_WIREFRAME : D3DFILL_SOLID);
			return 0;
		case ' ':
			animLoop=!animLoop;
			FrameCount-=FrameCount%3;
			return 0;
		case '.':
			if(!animLoop)FrameCount+=3;
			return 0;
		case ',':
			if(!animLoop)FrameCount-=3;
			return 0;
		case 13://enter
			if(FAILED(setScreen(TRUE)))
				ErrorQuit(E_FAIL,"Set Screen Error");
			return 0;
		}

	}

	return D3DAppBase::MsgProc(hWnd,uMsg,wParam,lParam);
}

INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR lpCmdLine, INT ){
	MD2Viewer theapp;
	theapp.lpCmdLine=lpCmdLine;
	theapp.WindowTitle=lpCmdLine;
	return SUCCEEDED(theapp.Create(hInst)) ? theapp.Run() : 1;
}

